/*
 * Script.cpp
 *
 *  Created on: Sep 19, 2012
 *      Author: tony
 */


#include "Script.h"
#include <./scriptstdstring/scriptstdstring.h>
#include <./scriptarray/scriptarray.h>
#include <./scriptnetburner/scriptnetburner.h>




Script::Script() {

	//myCircle.Print("hello from the circle\n");
	engine =0;
	ctx =0;
	errors =1;
	engine = asCreateScriptEngine(ANGELSCRIPT_VERSION);
	if( engine == 0 )
	{
		iprintf( "Failed to create script engine .\n");
		return;
	}

	 // Set the message callback to print the human readable messages that the engine gives in case of errors
	 engine->SetMessageCallback(asMETHOD(Script, ScriptMessageCallback), this, asCALL_THISCALL);

	// The script compiler will write any compiler messages to the callback.
	//engine->SetMessageCallback(asFUNCTION(ScriptMessageCallback), 0, asCALL_CDECL);

	//Register Arrays in script engine
	RegisterScriptArray(engine, true);
	//Register Strings in script engine
	RegisterStdString(engine);
	//Register stdio Printing in script engine
	RegisterPrint(engine);
	//Register PIN Digital IO in script engine
	RegisterPinDigitalIO(engine);
	//Register Timers in script engine
	RegisterTimers(engine);


	return;
}

Script::~Script() {
	// TODO Auto-generated destructor stub
}

void Script::ScriptMessageCallback(const asSMessageInfo &msg)
{
	char buffer [128];
	const char *type = "ERR ";
	if( msg.type == asMSGTYPE_WARNING )
		type = "WARN";
	else if( msg.type == asMSGTYPE_INFORMATION )
		type = "INFO";

	sprintf(buffer, "%s (%d, %d) : %s : %s \n", msg.section, msg.row, msg.col, type, msg.message);
	compileErrors += buffer;
}

int Script::CheckFunctions()
{
	std::vector<std::string> funcNames;
	std::vector<std::string>::iterator Iter;
	std::string Name;

	funcNames.push_back("void Init()");
	funcNames.push_back("void One_Hz()");
	funcNames.push_back("void Ten_Hz()");
	funcNames.push_back("void Hundred_Hz()");

	for( Iter = funcNames.begin(); Iter != funcNames.end(); Iter++)
	{
		Name= *Iter;
		asIScriptFunction  *funcId = engine->GetModule(0)->GetFunctionByDecl( Name.c_str());
		//int funcId = engine->GetModule(0)->GetFunctionIdByDecl("float calc(float, float)");

		if( funcId == NULL)
		{
			iprintf( "Function %s must be included in the script.\n", Name.c_str());
			ctx->Release();
			ctx = 0;
			engine->Release();
			engine = 0;
			return -1;
		}
	}

	return 1;
}

int Script::IsValid()
{
	if (errors!=0) {return 0;}
	else return 1;


}

void Script::Run(int Number)
{
	if (!IsValid()) return;
	char stepNum[128]={0,};
	sprintf(stepNum,"Step%d",Number);
	asIScriptFunction *functionID = methods[stepNum];
	RunFunction(functionID);
}

void Script::Run(char* Name)
{
	if (!IsValid()) return;
	asIScriptFunction * functionID = methods[Name];
    RunFunction(functionID);

}

void Script::RunFunction(asIScriptFunction  *fID)
{
	// Prepare the script context with the function we wish to execute. Prepare()
	// must be called on the context before each new script function that will be
	// executed. Note, that if you intend to execute the same function several
	// times, it might be a good idea to store the function id returned by
	// GetFunctionIDByDecl(), so that this relatively slow call can be skipped.

	r = ctx->Prepare(fID);
	if( r < 0 )
	{
		iprintf( "Failed to prepare the context.\n");
		ctx->Release();
		ctx =0;
		engine->Release();
		engine =0;
		return;
	}

	r = ctx->Execute();

	if( r != asEXECUTION_FINISHED )
	{
		// The execution didn't finish as we had planned. Determine why.
		if( r == asEXECUTION_ABORTED )
			iprintf( "The script was aborted before it could finish. Probably it timed out.\n");
		else if( r == asEXECUTION_EXCEPTION )
		{
			iprintf( "The script ended with an exception.\n");

			// Write some information about the script exception
			asIScriptFunction *func = ctx->GetExceptionFunction();
			iprintf( "func: %s\n", func->GetDeclaration());
			iprintf( "modl: %s\n", func->GetModuleName() );
			iprintf( "sect: %s\n" , func->GetScriptSectionName());
			iprintf( "line: %s\n" , ctx->GetExceptionLineNumber());
			iprintf( "desc: %s\n" , ctx->GetExceptionString());
		}
		else
			iprintf( "The script ended for some unforeseen reason %d \n" , r );
	}
	else
	{
		// Retrieve the return value from the context
		//float returnValue = ctx->GetReturnFloat();
		//std::cout << "The script function returned: " << returnValue << std::endl;
	}
}

void Script::Release()
{
	// We must release the contexts when no longer using them
	if(ctx)
	{
		iprintf( "Releasing the script context\n");
		ctx->Release();
	}

	// Release the engine
	if (engine)
	{
		iprintf( "Releasing the script engine\n");
		engine->Release();

	}
}

void Script::RegisterGlobalVariable(string declaration, void * variable)
{
	engine->RegisterGlobalProperty(declaration.c_str(), variable) ;
}


int Script::ConfigureEngine(string scriptText)
{
	errors =0;
	int funcCount=0;

	// Compile the script code
	r = CompileScript(scriptText);
	if( r < 0 )
	{
		return r;
		errors =1;
	}

	// We must release the contexts when no longer using them
	if(ctx)
	{
		iprintf( "Releasing the script context\n");
		ctx->Release();
		ctx = 0;
	}

	ctx = engine->CreateContext();
	if( ctx == 0 )
	{
		iprintf("Failed to create the context.\n");
		errors =1;
		return -1;
	}

	funcCount = engine->GetModule(0)->GetFunctionCount();
	iprintf( "The Function count is %d\n" , funcCount);
	for(int theCount =0 ; theCount < funcCount; theCount++)
	{
		asIScriptFunction *theFuncId = engine->GetModule(0)->GetFunctionByIndex(theCount);
		methods[theFuncId->GetName()] =   theFuncId; // engine->GetModule(0)->GetFunctionDescriptorById(theFuncId)->GetName()] =  theFuncId;
	}

	if(errors ==0) Run("Init");
	return 1;
	//return CheckFunctions();
}


int Script::CompileScript(string scriptData)
{
	int r;

	try
	{

		if( scriptData.size() == 0 )
		{
			iprintf( "Failed to load script.\n");
			return -1;
		}

		// Add the script sections that will be compiled into executable code.
		// If we want to combine more than one file into the same script, then
		// we can call AddScriptSection() several times for the same module and
		// the script engine will treat them all as if they were one. The script
		// section name, will allow us to localize any errors in the script code.

		asIScriptModule *mod = engine->GetModule(0, asGM_ALWAYS_CREATE);

		r = mod->AddScriptSection( "script", &scriptData[0],scriptData.size());
		if( r < 0 )
		{
			iprintf("AddScript script section() failed\n");
			return -1;
		}


		// Compile the script. If there are any compiler messages they will
		// be written to the message stream that we set right after creating the
		// script engine. If there are no errors, and no warnings, nothing will
		// be written to the stream.
		r = mod->Build();
		if( r < 0 )
		{
			iprintf( "Build() failed.\n");
			return -1;
		}

		// The engine doesn't keep a copy of the script sections after Build() has
		// returned. So if the script needs to be recompiled, then all the script
		// sections must be added again.

		// If we want to have several scripts executing at different times but
		// that have no direct relation with each other, then we can compile them
		// into separate script modules. Each module use their own namespace and
		// scope, so function names, and global variables will not conflict with
		// each other.
	}
	catch(...)
	{
		iprintf("!!!! An unknown error occurred while building the script.\n");
	}


	return 0;
}

