#include "scriptnetburner.h"
#include <assert.h> // assert()

using namespace std;

BEGIN_AS_NAMESPACE


/*-------------------------------------------------------------------------------
	Register timer function to allow the script to sleep or delay
--------------------------------------------------------------------------------*/

void OSTimeDly_Generic(asIScriptGeneric *gen)
{
	int ticks = *(int*)gen->GetAddressOfArg(0);
	OSTimeDly(ticks);
}

//This is a high resolution delay
void hrOSTimeDly_Generic(asIScriptGeneric *gen)
{
	HiResTimer *timer;
	double seconds = *(double*)gen->GetAddressOfArg(0);
	timer = HiResTimer::getHiResTimer();
	timer->delay(seconds);
}




void RegisterTimers_Generic(asIScriptEngine *engine)
{
	int r;
	//This is a low resolution delay
	r = engine->RegisterGlobalFunction("void OSTimeDly(int ticks)", asFUNCTION(OSTimeDly_Generic), asCALL_GENERIC);  assert( r >= 0 );
	//This is a high resolution delay
	r = engine->RegisterGlobalFunction("void hrOSTimeDly(double seconds)", asFUNCTION(hrOSTimeDly_Generic), asCALL_GENERIC);  assert( r >= 0 );
}

void RegisterTimers_Native(asIScriptEngine *engine)
{
	//Native not currently supported
}


void RegisterTimers(asIScriptEngine *engine)
{
	if (strstr(asGetLibraryOptions(), "AS_MAX_PORTABILITY"))
		RegisterTimers_Generic(engine);
	else
		RegisterTimers_Native(engine);
}


/*-------------------------------------------------------------------------------
	Register digital IO functions to set and reset pins
--------------------------------------------------------------------------------*/

void setPin_Generic(asIScriptGeneric *gen)
{
	int pinNum = *(int*)gen->GetAddressOfArg(0);

	J2[pinNum] =1;
}
void resetPin_Generic(asIScriptGeneric *gen)
{
	int pinNum = *(int*)gen->GetAddressOfArg(0);

	J2[pinNum] =0;
}

void RegisterPinDigitalIO_Generic(asIScriptEngine *engine)
{
	int r;
	// Register setPin function
	r = engine->RegisterGlobalFunction("void setPin(int pinNum)", asFUNCTION(setPin_Generic), asCALL_GENERIC);  assert( r >= 0 );
	// Register resetPin function
	r = engine->RegisterGlobalFunction("void resetPin(int pinNum)", asFUNCTION(resetPin_Generic), asCALL_GENERIC);  assert( r >= 0 );


}

void RegisterPinDigitalIO_Native(asIScriptEngine *engine)
{
	//Native not currently supported
}


void RegisterPinDigitalIO(asIScriptEngine *engine)
{
	if (strstr(asGetLibraryOptions(), "AS_MAX_PORTABILITY"))
		RegisterPinDigitalIO_Generic(engine);
	else
		RegisterPinDigitalIO_Native(engine);
}


/*-------------------------------------------------------------------------------
	Register standard printf stdio through serial port
--------------------------------------------------------------------------------*/

void PrintString_Generic(asIScriptGeneric *gen)
{
	 string * str = static_cast<string *>(gen->GetArgObject(0));
	 printf(str->c_str());
}

void RegisterPrint_Generic(asIScriptEngine *engine)
{
	int r;
	// Register the print function
	r = engine->RegisterGlobalFunction("void print(string &in)", asFUNCTION(PrintString_Generic), asCALL_GENERIC); assert( r >= 0 );

}

void RegisterPrint_Native(asIScriptEngine *engine)
{
	//Native not currently supported
}


void RegisterPrint(asIScriptEngine *engine)
{
	if (strstr(asGetLibraryOptions(), "AS_MAX_PORTABILITY"))
		RegisterPrint_Generic(engine);
	else
		RegisterPrint_Native(engine);
}

END_AS_NAMESPACE
